@@ -1,40 +1,55 @@
-cv.sdwd = function(x, y, lambda=NULL, nfolds=5, foldid, ...) {
-  N = nrow(x)
-  ## Fit the model once to get dimensions etc of output
-  y = drop(y)
+cv.sdwd = function(x, y, lambda=NULL, 
+  pred.loss=c("misclass", "loss"), nfolds=5, foldid, ...) {
+  ####################################################################
+  ## data setup
+  if (missing(pred.loss)) 
+    pred.loss = "misclass" else pred.loss = match.arg(pred.loss)
+  if (!match(pred.loss, c("misclass", "loss"), FALSE)) {
+    warning("Only 'misclass' and 'loss' available for 
+            DWD; 'misclass' used.")
+    pred.loss = "misclass"
+  }
+  typenames = c(misclass = "mis-classification error",
+                loss     = "DWD loss")
+  y = c(-1, 1)[as.factor(drop(y))]
+  x = as.matrix(x)
+  if (!all(y %in% c(-1, 1))) 
+    stop("y should be a factor with two levels.")
+  x.row = as.integer(NROW(x))
+  if (length(y) != x.row) 
+    stop("x and y have different number of observations.")  
+  ####################################################################
   sdwd.object = sdwd(x, y, lambda=lambda, ...)
   lambda = sdwd.object$lambda
-  # predict -> coef
   nz = sapply(coef(sdwd.object, type="nonzero"), length)
   if (missing(foldid)) 
     foldid = sample(rep(seq(nfolds), 
-      length=N)) else nfolds = max(foldid)
+      length=x.row)) else nfolds = max(foldid)
   if (nfolds < 3) 
-    stop("nfolds must be bigger than 3; nfolds=10 recommended")
+    stop("nfolds must be bigger than 3; nfolds=5 recommended.")
   outlist = as.list(seq(nfolds))
-  ## Now fit the nfold models and store them
+  ## fit the model nfold times and save them
   for (i in seq(nfolds)) {
     which = foldid == i
     outlist[[i]] = sdwd(x=x[!which, , drop=FALSE], 
                         y=y[!which], lambda=lambda, ...)
   }
-  ## What to do depends on the model fit
-  pred.loss = "misclass"
+  ## select the lambda according to predmat
   fun = paste("cv", class(sdwd.object)[[2]], sep=".")
-  cvstuff = do.call(fun, list(outlist, lambda, x, y, foldid))
+  cvstuff = do.call(fun, 
+    list(outlist, lambda, x, y, foldid, pred.loss))
   cvm = cvstuff$cvm
   cvsd = cvstuff$cvsd
   cvname = cvstuff$name
   out = list(lambda=lambda, cvm=cvm, cvsd=cvsd, 
               cvupper=cvm+cvsd, cvlower=cvm - cvsd, nzero=nz, 
               name=cvname, sdwd.fit=sdwd.object)
-  lamin = getmin(lambda, cvm, cvsd)
-  obj = c(out, as.list(lamin))
+  obj = c(out, as.list(getmin(lambda, cvm, cvsd)))
   class(obj) = "cv.sdwd"
   obj
 } 
 
-cv.sdwdNET = function(outlist, lambda, x, y, foldid) {
+cv.sdwdNET = function(outlist, lambda, x, y, foldid, pred.loss) {
   ### Turn y into c(-1,1)
   y = as.factor(y)
   y = c(-1, 1)[as.numeric(y)]
@@ -49,18 +64,19 @@ cv.sdwdNET = function(outlist, lambda, x, y, foldid) {
     predmat[which, seq(nlami)] = preds
     nlams[i] = nlami
   }
-  pred.loss="misclass"
-  typenames = c(misclass = "Mis-classification Error")
-  N = length(y) - apply(is.na(predmat),2,sum)
+  typenames = c(misclass = "mis-classification error",
+              loss     = "DWD loss")
   cvraw = switch(pred.loss, 
+    loss     = dwdloss(y * predmat),
     misclass = (y != ifelse(predmat > 0, 1, -1)))
   if (length(y)/nfolds >= 3) {
     cvob = cvcompute(cvraw, foldid, nlams)
-    cvraw = cvob$cvraw; N = cvob$N
-  }     
-  cvm = apply(cvraw, 2, mean, na.rm=TRUE)
-  cvsd = sqrt(apply(scale(cvraw, cvm, FALSE)^2, 2, 
-    mean, na.rm=TRUE)/(N - 1))
+    cvraw = cvob$cvraw
+    cvn = cvob$N
+  } else cvn = length(y) - colSums(is.na(predmat))    
+  cvm = colMeans(cvraw, na.rm=TRUE)
+  cvsd = sqrt(colMeans(scale(cvraw, cvm, FALSE)^2, 
+    na.rm=TRUE)/(cvn - 1))
   list(cvm = cvm, cvsd = cvsd, name = typenames[pred.loss])
 } 
 
@@ -74,15 +90,14 @@ predict.cv.sdwd = function(object, newx, s=c("lambda.1se",
   predict(object$sdwd.fit, newx, s=lambda, ...)
 } 
 
-
 plot.cv.sdwd = function(x, sign.lambda=1, ...) {
   cvobj = x
   xlab = "log(Lambda)"
   if (sign.lambda < 0) 
     xlab = paste0("-", xlab)
-  plot.args = list(x = sign.lambda * log(cvobj$lambda), 
-    y = cvobj$cvm, type = "n", xlab = xlab, ylab = cvobj$name,
-    ylim = range(cvobj$cvupper, cvobj$cvlo))
+  plot.args = list(x=sign.lambda * log(cvobj$lambda), 
+    y=cvobj$cvm, type="n", xlab=xlab, ylab=cvobj$name,
+    ylim=range(cvobj$cvupper, cvobj$cvlo))
   new.args = list(...)
   if (length(new.args)) 
     plot.args[names(new.args)] = new.args
@@ -98,8 +113,8 @@ plot.cv.sdwd = function(x, sign.lambda=1, ...) {
   invisible()
 } 
 
-coef.cv.sdwd = function(object, s=c("lambda.1se", 
-    "lambda.min"), ...) {
+coef.cv.sdwd = function(object, 
+    s=c("lambda.1se", "lambda.min"), ...) {
   if (is.numeric(s)) 
     lambda = s else if (is.character(s)) {
       s = match.arg(s)
